require 'spec_helper'
require 'strain/schema_tools/defaults.rb'

module Strain::SchemaTools
  RSpec.describe Defaults do
    subject(:d) { Defaults.new(schema, data) }
    let(:schema) do
      { "type" => "object", "properties" => {
          "title" => {"type" => "string", "default" => "default title"},
          "summary" => {"type" => "string"},
          "author" => {
            "type" => "object", "properties" => {
              "name" => {"type" => "string", "default" => "default author"},
              "date" => {"type" => "integer", "default" => 2005}
      } } } }
    end
    let(:data) do
      { "title" => nil,
        "summary" => "test summary",
        "author" => {
          "name" => "Jane Author",
          "date" => nil
      } }
    end

    describe "#default_values" do
      subject(:default_values) {d.default_values}

      context "with a simple schema" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "title" => {"type" => "string", "default" => "default title"}
          } }
        end
        it "returns a hash with all declared defaults" do
          expect(default_values).to eq({"title" => "default title"})
        end
      end

      context "with a typical schema" do
        it "returns a hash with all declared defaults" do
          expect(default_values).to eq(
            {"title"=>"default title", "author"=>{"name"=>"default author", "date"=>2005}}
          )
        end
      end

      context "with a nested schema" do
        let(:schema) do
          {
            "type" => "object", "properties" => {
            "title" => {"type" => "string", "default" => "default title"},
            "summary" => {"type" => "string"},
            "author" => {
              "type" => "object", "properties" => {
                "name" => {"type" => "string", "default" => "default author"},
                "date" => {"type" => "integer", "default" => 2005}
          } } } }
        end
        it "returns a hash with all declared defaults" do
          expect(default_values).to eq({
            "title" => "default title",
            "author" => {
              "name" => "default author",
              "date" => 2005
          } })
        end
      end

      context "with array defaults" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "numbers" => {"type" => "array", "default" => "[1, 2, 3]"}
          } }
        end
        it "returns a hash with all declared defaults" do
          expect(default_values).to eq({"numbers" => "[1, 2, 3]"})
        end
      end

      context "with non-conflicting object defaults" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "obj_with_defaults" => {
              "type" => "object",
              "default" => {"inner_prop" => "test"},
              "properties" => {"inner_prop" => { "type": "string" }}
          } } }
        end
        it "returns a hash with all declared defaults" do
          expect(default_values).to eq(
            {"obj_with_defaults" => {"inner_prop" => "test"}}
          )
        end
      end

      context "with conflicting object defaults" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "obj_with_defaults" => {
              "type" => "object",
              "default" => {"inner_prop" => "outer_default"},
              "properties" => {
                "inner_prop" => {"type" => "string", "default" => "inner_default"}
          } } } }
        end
        it "prioritizes inner defaults" do
          expect(default_values).to eq(
            {"obj_with_defaults" => {"inner_prop" => "inner_default"}}
          )
        end
      end

      context "with no properties on an object and no defaults" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "obj_with_defaults" => {
              "type" => "object"
          } } }
        end
        it "returns a blank hash" do
          expect(default_values).to eq({})
        end
      end

      context "with no properties on an object and defaults" do
        let(:schema) do
          { "type" => "object", "properties" => {
            "obj_with_defaults" => {
              "type" => "object",
              "default" => {"inner_prop" => "test"}
          } } }
        end
        it "returns a hash with defaults set" do
          expect(default_values).to eq({"obj_with_defaults" => {"inner_prop" => "test"}})
        end
      end
    end

    describe "#data_with_defaults" do
      subject(:data_with_defaults) {d.data_with_defaults}

      context "with some missing data and basic schema" do
        it "merges defaults with the data" do
          expect(data_with_defaults).to eq({
            "title" => "default title",
            "summary" => "test summary",
            "author" => {
              "name" => "Jane Author",
              "date" => 2005
          } })
        end
      end

      context "with blank strings" do
        let(:data) do
          { "title" => "",
            "summary" => "",
            "author" => ""
          }
        end
        it "replaces blank strings with defaults" do
          expect(data_with_defaults).to eq({
            "title" => "default title",
            "summary" => "",
            "author" => {
              "name" => "default author",
              "date" => 2005
          } })
        end
      end

      context "with blank arrays" do
        let(:data) do
          {"numbers" => []}
        end
        let(:schema) do
          { "type" => "object",
            "properties" => {
              "numbers" => {"type" => "array", "default" => [1,2,3]}
          } }
        end
        it "replaces blank arrays with defaults" do
          expect(data_with_defaults).to eq({
            "numbers" => [1,2,3]
          })
        end
      end

      context "with blank arrays containing objects" do
        let(:data) do
          {"tags" => []}
        end
        let(:schema) do
          { "type" => "object",
            "properties" => {
              "tags" => {
                "type" => "array",
                "items" => {"type" => "string"},
                "default" => [{"tag1" => "red"}, {"tag2"=>"green"}]
          } } }
        end
        it "replaces blank arrays with defaults" do
          expect(data_with_defaults).to eq({
            "tags" => [{"tag1" => "red"}, {"tag2"=>"green"}]
          })
        end
      end

      context "with deeply nested objects" do
        let(:data) do
          {"second_level" => {} }
        end
        let(:schema) do
          { "type" => "object",
            "properties" => {
              "second_level" => {
                "type" => "object",
                "properties" => {
                  "third_level" => {
                    "type" => "object",
                    "default" => {"deep_property" => "test"}
          } } } } }
        end
        it "merges defaults" do
          expect(data_with_defaults).to eq({
            "second_level"=>{"third_level"=>{"deep_property"=>"test"}}
          })
        end
      end
    end

  end
end
