require 'rails_helper'

RSpec.describe "Articles", type: :request do

  describe "Index / GET collection" do

    before(:all) do
      create :article
      get jsonapi_variants_path
    end

    after(:all) do
      Strain::Article.delete_all
      Strain::Variant.delete_all
    end

    it "returns HTTP 200" do
      expect(response).to have_http_status(200)
    end

    it "responds with JSONAPI media type" do
      expect(response.header["Content-Type"]).to eq("application/vnd.api+json")
    end

    it "returns a list of Articles" do
      data = JSON.parse(response.body).fetch("data")
      expect(data.size).to eq(Strain::Article.all.count)
    end

  end

  describe "Show / GET member" do

    before(:all) do
      @article = create(:article)
      get jsonapi_article_path(@article)
      @body = JSON.parse(response.body)
    end
    after(:all) do
      Strain::Article.delete_all
      Strain::Variant.delete_all
    end

    it "returns HTTP 200" do
      expect(response).to have_http_status(200)
    end

    it "responds with JSONAPI media type" do
      expect(response.header["Content-Type"]).to eq("application/vnd.api+json")
    end

    it "responds with the created article" do
      returned = @body.fetch("data")
      expect(returned["type"]).to eq("articles")
      expect(returned["attributes"]).to have_key("slug")
      expect(returned["attributes"]).to have_key("title")
      # expect(returned["attributes"]).to have_key("variant")
      expect(returned["attributes"]).to have_key("body")
      expect(returned["id"]).to eq("#{@article.id}")
    end

  end

  describe "Create / POST collection" do

    context "when request is valid" do
      before(:all) do
        @variant = create :variant
        headers = {
          "Accept": "application/vnd.api+json",
          "Content-Type": "application/vnd.api+json"
        }
        request_body = JSON.dump({
          "data" => {
            "type" => "articles",
            "attributes" => {
              "title" => "test title",
              "slug" => "test-slug",
              "body" => {
                "summary" => "test_summary",
                "main" => "test_main",
              }
            },
            "relationships" => {
              "variant" => {
                "data" => {"type" => "variants", "id" => @variant.id}
              }
            }
          }
        })
        post jsonapi_articles_path, request_body, headers
        @body = JSON.parse(response.body)
      end
      after(:all) do
        Strain::Variant.delete_all
        Strain::Article.delete_all
      end

      it "returns HTTP 201(Created)" do
        expect(response).to have_http_status(201)
      end

      it "returns the created Article" do
        returned = @body.fetch("data")
        expect(returned["type"]).to eq("articles")
        expect(returned["attributes"]["body"]["summary"]).to eq("test_summary")
        expect(returned["attributes"]["body"]["main"]).to eq("test_main")
      end
    end

    context "when request is invalid" do
      before(:all) do
        @variant = create :variant
        headers = {
          "Accept": "application/vnd.api+json",
          "Content-Type": "application/vnd.api+json"
        }
        request_body = JSON.dump({
          "data" => {
            "type" => "articles",
            "attributes" => {
              "title" => "test title",
              "slug" => "test-slug",
              "body" => {
                "summary" => "test_summary",
                "main" => "test_main",
                "extra_propert" => "test"
              }
            },
            "relationships" => {
              "variant" => {
                "data" => {"type" => "variants", "id" => @variant.id}
              }
            }
          }
        })
        post jsonapi_articles_path, request_body, headers
        @body = JSON.parse(response.body)
      end
      after(:all) do
        Strain::Variant.delete_all
        Strain::Article.delete_all
      end

      it "returns HTTP 422(unprocessable entity)" do
        expect(response).to have_http_status(422)
      end

      it "returns an error hash" do
        returned = @body.fetch("errors")
        expect(returned).not_to be_empty
      end
    end

  end

  describe "Destroy / DELETE member" do

    before(:all) do
      @article = create :article
      delete jsonapi_article_path(@article)
    end
    after(:all) do
      Strain::Variant.delete_all
      Strain::Article.delete_all
    end

    it "returns HTTP 204(successful/no content)" do
      expect(response).to have_http_status(204)
    end

    it "deletes the Article" do
      expect(Strain::Article.count).to eq(0)
    end

  end

  describe "Update / PATCH member" do

    before(:all) do
      @article = create :article
      headers = {
        "Accept": "application/vnd.api+json",
        "Content-Type": "application/vnd.api+json"
      }
      request_body = JSON.dump({
        "data": {
          "id": @article.id,
          "type": "articles",
          "attributes": {
            "title": "updated"
          }
        }
      })
      patch jsonapi_article_path(@article), request_body, headers
    end
    after(:all) do
      Strain::Variant.delete_all
      Strain::Article.delete_all
    end

    it "returns HTTP 200(successful)" do
      expect(response).to have_http_status(200)
    end

    it "returns the updated Article in the response body" do
      returned = JSON.parse(response.body).fetch("data")
      expect(returned["type"]).to eq("articles")
      expect(returned["attributes"]["title"]).to eq("updated")
    end

    it "updates the Article" do
      updated = Strain::Article.find(@article.id)
      expect(updated.title).to eq("updated")
      expect(@article.slug).to eq(updated.slug)
    end

  end

end
