require 'rails_helper'

RSpec.describe "Variants", type: :request do

  describe "Index / GET collection" do

    context "when authenticated" do

      before(:all) do
        create :variant
        get jsonapi_variants_path
      end

      after(:all) {Strain::Variant.delete_all}

      it "returns HTTP 200" do
        expect(response).to have_http_status(200)
      end

      it "responds with JSONAPI media type" do
        expect(response.header["Content-Type"]).to eq("application/vnd.api+json")
      end

      it "returns a list of Variants" do
        data = JSON.parse(response.body).fetch("data")
        expect(data.size).to eq(Strain::Variant.all.count)
      end
    end
  end

  describe "Show / GET member" do
    context "when authenticated" do

      before(:all) do
        @variant = create(:variant)
        get jsonapi_variant_path(@variant)
        @body = JSON.parse(response.body)
      end
      after(:all) {Strain::Variant.delete_all}

      it "returns HTTP 200" do
        expect(response).to have_http_status(200)
      end

      it "responds with JSONAPI media type" do
        expect(response.header["Content-Type"]).to eq("application/vnd.api+json")
      end

      it "responds with the created Variant" do
        returned = @body.fetch("data")
        expect(returned["type"]).to eq("variants")
        expect(returned["attributes"]).to have_key("name")
        expect(returned["attributes"]).to have_key("schema")
        expect(returned["id"]).to eq("#{@variant.id}")
      end

    end
  end

  describe "Create / POST collection" do
    context "when authenticated" do

      before(:all) do
        request_body = JSON.dump({
          "data": {
            "type": "variants",
            "attributes": {
              "name": "test",
              "schema": {"type": "object"}
            }
          }
        })

        headers = {
          "Accept": "application/vnd.api+json",
          "Content-Type": "application/vnd.api+json"
        }

        post jsonapi_variants_path, request_body, headers
        @body = JSON.parse(response.body)
      end
      after(:all) {Strain::Variant.delete_all}

      it "returns HTTP 201(Created)" do
        expect(response).to have_http_status(201)
      end

      it "returns the created Variant" do
        returned = @body.fetch("data")
        expect(returned["type"]).to eq("variants")
        expect(returned["attributes"]["name"]).to eq("test")
      end

    end
  end

  describe "Destroy / DELETE member" do
    context "when authenticated" do
      before(:all) do
        @variant = create :variant
        delete jsonapi_variant_path(@variant)
      end
      after(:all) do
        Strain::Variant.delete_all
      end

      it "returns HTTP 204(successful/no content)" do
        expect(response).to have_http_status(204)
      end

      it "deletes the Variant" do
        expect(Strain::Variant.all.count).to eq(0)
      end

    end
  end

  describe "Update / PATCH member" do
    context "when authenticated" do

      before(:all) do
        @variant = create :variant
        headers = {
          "Accept": "application/vnd.api+json",
          "Content-Type": "application/vnd.api+json"
        }
        request_body = JSON.dump({
          "data": {
            "id": @variant.id,
            "type": "variants",
            "attributes": {
              "name": "updated"
            }
          }
        })
        patch jsonapi_variant_path(@variant), request_body, headers
      end
      after(:all) { Strain::Variant.delete_all }

      it "returns HTTP 200(successful)" do
        expect(response).to have_http_status(200)
      end

      it "returns the updated Variant in the response body" do
        returned = JSON.parse(response.body).fetch("data")
        expect(returned["type"]).to eq("variants")
        expect(returned["attributes"]["name"]).to eq("updated")
      end

      it "updates the Variant" do
        updated = Strain::Variant.find(@variant.id)
        expect(updated.name).to eq("updated")
        expect(@variant.schema).to eq(updated.schema)
      end

    end
  end

end
